@@ -1,3 +1,10 @@
+import sys
+if len(sys.argv) > 1 and sys.argv[1] == "--usegevent":
+    sys.argv.pop(1)
+    from gevent import monkey
+    monkey.patch_all()
+
+
 from .core import (
     action,  # main py4web decorator
     request,
@@ -19,4 +26,4 @@ from .core import (
 
 __author__ = "Massimo Di Pierro <massimo.dipierro@gmail.com>"
 __license__ = "BSDv3"
-__version__ = "1.20210119.1"
+__version__ = "1.20210207.1"
